% driver to generate plot for example 2.5.4 on page 229

disp('Numerical experiments for splitting schemes');
% General parameters
tspan = [0,1];
lambda = 1;
y0 = 0;

% Right hand side
f = @(t,x) lambda*x*(1-x)+sqrt(1-x^2);

disp('Compute almost exact solution');

% First step: compute very accurate solution
options = odeset('reltol',1.0e-10,'abstol',1.0e-12,'stats','on');
[t,yex] = ode45(f,[0,1],y0,options);

% experiment (Example ex:splitcvg)
% Comparison of splitting schemes
disp('Comparison of splitting methods based on discrete timestepping');
err = [];
for N=10:10:200
  fprintf('N=%d\n',N);
  % function handles to ESVs
  psisin = @(h,y) sinsolve(h,y);
  psi1 = @(h,y) logeeul(h,y,lambda);
  psi2 = @(h,y) sineeul(h,y);
  psi3 = @(h,y) logieul(h,y,lambda);
  psi4 = @(h,y) logemp(h,y,lambda);
  psi5 = @(h,y) sinemp(h,y);
  
  % Lie-Trotter and Explicit Euler
  [t,ylt] = lietrotter(psi1,psi2,y0,tspan,N);
  errlt = abs(ylt(end)-yex(end));
  
  % Strang and Explicit Euler
  [t,yst] = strang(psi1,psi2,psi1,y0,tspan,N);
  errst = abs(yst(end)-yex(end));
  
  % Strang with Explcit Euler,exact, Implicit Euler
  [t,ysts] = strang(psi1,psisin,psi3,y0,tspan,N);
  errsts = abs(ysts(end)-yex(end));
  
  % Lie-Trotter and explicit midpoint
  [t,ylt] = lietrotter(psi4,psi5,y0,tspan,N);
  errltmp = abs(ylt(end)-yex(end));
  
  % Strang and implicit midpoint
  [t,yst] = strang(psi4,psi5,psi4,y0,tspan,N);
  errstmp = abs(yst(end)-yex(end));
  
  err = [err;1/N,errlt,errst,errsts,errltmp,errstmp];
end

figure('name','convergence of approximate splitting schemes');
loglog(err(:,1),err(:,2),'r-+',err(:,1),err(:,3),'b-+',...
       err(:,1),err(:,4),'k-^',err(:,1),err(:,5),'m-^',...
       err(:,1),err(:,6),'c-s');
axis([1/300 1/5 8*10^(-7) 2*10^(-2)]);
set(gca,'fontsize',14);
xlabel('Zeitschrittweite h');
ylabel('|y(T)-y_h(T)|');
legend('LTS-Eul','SS-Eul','SS-EuEI','LTS-EMP','SS-EMP',4);
print -dpsc2 'splitinex.eps';


